/* Framework for Polygon Structures on Clusters
 *
 * Leif Christian Larsen <leifchl@idi.ntnu.no>
 *
 */

/* fw_main.h: Header file for main program/utility routines
 */

#ifndef FW_MAIN
#define FW_MAIN

#include <stdio.h>
#include <stdlib.h>
#include <assert.h>

/* fw_realloc: Reallocate @ptr to total size @size.
 * Returns a pointer to new area with @size bytes of allocated memory.
 */

static void	*fw_realloc(void *ptr, size_t size)
{
	void *result;

	// No assert (result != NULL) here since realloc may well return NULL if size=0.

	result = realloc(ptr, size);

	return result;
}

/* fw_malloc: Malloc wrapper 
 */

static void	*fw_malloc(size_t size)
{
	void *result;

	/* It is convenient in some algorithms to use malloc(0).
	 */

	if(size <= 0)
		size = 1;

	result = malloc(size);

	assert(result != NULL);

	return result;
}

/* fw_free: free() wrapper
 */

static void	fw_free(void *ptr)
{
	assert(ptr != NULL);
	free(ptr);
}


void	printf_zero(const char *fmt, ...);

#endif
