/* Framework for Polygon Structures on Clusters
 *
 * Leif Christian Larsen <leifchl@idi.ntnu.no>
 *
 */
/* fw_quaternion.h: Header file for quaternion structure
 */

#ifndef FW_QUATERNION
#define FW_QUATERNION

#include "fw_vector.h"
#include "fw_quaternion.h"

enum qtypes { Q_NORMAL, Q_COMBINED };
enum qcache { QC_INVALID, QC_VALID };

typedef struct quaternion {
	enum qtypes qtype;
	vector v; 			// vector
	double s; 			// parameter
	double q[4]; 			// actual quaternion 
	double mtx[16]; 		// 4x4 rotation matrix 
	enum qcache qcachestatus; 	// is rotation matrix updated? 
} quaternion;


/* See fw_quaternion.c for further information on these functions
 */

static void		quaternion_set_angle(quaternion *q);

quaternion		quaternion_create(vector v, double s);
void			quaternion_normalize(quaternion *q);
void			quaternion_rotate(quaternion *q, double angle);
void			quaternion_invert(quaternion *q);
quaternion		quaternion_combine(quaternion *qa, quaternion *qb);
void			quaternion_update_matrix(quaternion *q);

#endif
