/* Framework for Polygon Structures on Clusters
 *
 * Leif Christian Larsen <leifchl@idi.ntnu.no>
 *
 */

/* fw_voxelcachetransfer.h: The Bounding Volumes communication
 * rotuines. See fw_voxelcachetransfer.c
 */

#ifndef FW_VOXELCACHETRANSFER
#define FW_VOXELCACHETRANSFER

#include "fw_voxel.h"

// Receive queue structure, one for each node on every node

typedef struct voxelcache_receivequeue {
	int recvrequests;			// # of receive requests in flight
	signed char *recvfromnodes;		// recvfromnodes[i] signals that we are going to receive from node i
	int *recvlengths;			// recvlengths[i] = amount of data to be received from node i
	cached_voxel **recvbufs;		// receive buffers; recvbufs[i] = array of recvlengths[i] cached_voxels
	MPI_Request *requests;			// array of MPI_Requests for receives from each node
} voxelcache_receivequeue;

// Send queue structure

typedef struct voxelcache_sendbuf {
	int size;				// # of elements in buffer
	int used;				// # of elements actually used (see fw_voxelcachetransfer.c/sendqueue_add)
	signed char sendexpect; 		// does the node expect to receive voxels?
	cached_voxel *buffer;			// actual buffer
	MPI_Request lenrequest;			// length request (see fw_voxelcachetransfer.c/exchange_start)
	MPI_Request datarequest;		// data request (see fw_voxelcachetransfer.c/exchange_start)
} voxelcache_sendbuf;

// The following structure wraps an array of sendbuffers, one for each node

typedef struct voxelcache_sendqueue {
	voxelcache_sendbuf *sendbuffers;
} voxelcache_sendqueue;


void				voxelcache_resetcachemap(native_voxel_space *nvs);

voxelcache_receivequeue  	*voxelcache_receivequeue_create(void);
void				voxelcache_receivequeue_free(voxelcache_receivequeue *rq);


voxelcache_sendqueue     	*voxelcache_sendqueue_create(void);
void				voxelcache_sendqueue_free(voxelcache_sendqueue *sq);
void				voxelcache_sendqueue_add(voxelcache_sendqueue *sq, coordinate min, coordinate max, int rrank);

void				voxelcache_exchange_start(voxelcache_receivequeue *rq, voxelcache_sendqueue *sq);
void				voxelcache_exchange_finalize(voxelcache_receivequeue *rq, voxelcache_sendqueue *sq);



#endif
