/* Framework for Polygon Structures on Clusters
 *
 * Leif Christian Larsen <leifchl@idi.ntnu.no>
 *
 */

/* fw_voxeltransfer.h: The Minimum Transfer/Blocked Transfer communication
 * rotuines. See fw_transfer.c
 */
#ifndef FW_VOXELTRANSFER
#define FW_VOXELTRANSFER

#include "fw_voxel.h"
#include "fw_voxelcachetransfer.h"

// Receive queue structure, one for each node on every node

typedef struct voxeltransfer_receivequeue {
	int recvrequests;			// # of receive requests in flight
	signed char *recvfromnodes;		// recvfromnodes[i] signals that we are going to receive from node i, also used for flags (see .c file)
	int *recvlengths;			// recvlengths[i] = amount of data to be received from node i
	float **recvbufs_voxels;		// receive buffers; recvbufs_voxels is the voxel buffer and recvbufs_coords is the coordinate buffer
	coordinate **recvbufs_coords;		// note that the coordinate buffer contains the requests FROM other nodes, but the voxel buffer contains the 
						// voxels RECEIVED from other nodes.
	MPI_Request *coordrequests;		// coordinate requests
	MPI_Request *voxelrequests;		// voxelrequests

} voxeltransfer_receivequeue;

// Send queue structure

typedef struct voxeltransfer_sendbuf {
	int size;				// # of elements in buffer
	int used;				// # of elements actually used (see fw_voxeltransfer.c/sendqueue_add/sendqueue_add_blocked)
	signed char sendexpect; 		// does the node expect to receive voxels?
	float *buffer_v;			// voxel send buffers; these are the voxels a node has requested and which will be sent to that node
	coordinate *buffer_c;			// coordinates requested FROM a node. this request buffer will be sent to the node as requests for voxels.
	MPI_Request lengthrequest;		// MPI_Request objects for length, coordinate and voxel sends-
	MPI_Request coordrequest;
	MPI_Request voxelrequest;
} voxeltransfer_sendbuf;

// The following structure wraps an array of sendbuffers, one for each node

typedef struct voxeltransfer_sendqueue {
	voxeltransfer_sendbuf *sendbuffers;
} voxeltransfer_sendqueue;


voxeltransfer_sendqueue 		*voxeltransfer_sendqueue_create(void);
void					voxeltransfer_sendqueue_free(voxeltransfer_sendqueue *sq);
void					voxeltransfer_sendqueue_add_coordinate(voxeltransfer_sendqueue *sq, coordinate *c, int rrank);
void					voxeltransfer_sendqueue_add_coordinate_blocked(voxeltransfer_sendqueue *sq, coordinate *c, int rrank, voxel_cache *tmpvoxelcache);


voxeltransfer_receivequeue 		*voxeltransfer_receivequeue_create(void);
void					voxeltransfer_receivequeue_free(voxeltransfer_receivequeue *rq);
void					voxeltransfer_exchange(voxeltransfer_receivequeue *rq, voxeltransfer_sendqueue *sq);


#endif
